@@ -15,6 +15,7 @@ trillium_smol::config() // or trillium_async_std, trillium_tokio
     .with_port(8080) // the default
     .with_host("localhost") // the default
     .with_nodelay()
+    .with_max_connections(Some(10000))
     .without_signals()
     .run(|conn: trillium::Conn| async move { conn.ok("hello") });
 ```
@@ -29,6 +30,11 @@ The socket binding logic is as follows:
   * Host will be selected from explicit configuration using
     [`Config::with_host`] or else the `HOST` environment variable,
     or else a default of "localhost".
+    * On `cfg(unix)` systems only: If the host string (as set by env var
+      or direct config) begins with `.`, `/`, or `~`, it is
+      interpreted to be a path, and trillium will bind to it as a unix
+      domain socket. Port will be ignored. The socket will be deleted
+      on clean shutdown.
   * Port will be selected from explicit configuration using
     [`Config::with_port`] or else the `PORT` environment variable,
     or else a default of 8080.
@@ -48,7 +54,7 @@ In order to use this to _implement_ a trillium server, see
 */
 
 #[derive(Debug)]
-pub struct Config<ServerType, AcceptorType> {
+pub struct Config<ServerType: ?Sized, AcceptorType> {
     pub(crate) acceptor: AcceptorType,
     pub(crate) port: Option<u16>,
     pub(crate) host: Option<String>,
@@ -56,12 +62,13 @@ pub struct Config<ServerType, AcceptorType> {
     pub(crate) stopper: Stopper,
     pub(crate) counter: CloneCounter,
     pub(crate) register_signals: bool,
+    pub(crate) max_connections: Option<usize>,
     server: PhantomData<ServerType>,
 }
 
 impl<ServerType, AcceptorType> Config<ServerType, AcceptorType>
 where
-    ServerType: Server,
+    ServerType: Server + ?Sized,
     AcceptorType: Acceptor<ServerType::Transport>,
 {
     /// Starts an async runtime and runs the provided handler with
@@ -129,6 +136,7 @@ where
             stopper: self.stopper,
             counter: self.counter,
             register_signals: self.register_signals,
+            max_connections: self.max_connections,
         }
     }
 
@@ -137,6 +145,16 @@ where
         self.stopper = stopper;
         self
     }
+
+    /**
+    Configures the maximum number of connections to accept. The
+    default is 75% of the soft rlimit_nofile (`ulimit -n`) on unix
+    systems, and None on other sytems.
+    */
+    pub fn with_max_connections(mut self, max_connections: Option<usize>) -> Self {
+        self.max_connections = max_connections;
+        self
+    }
 }
 
 impl<ServerType> Config<ServerType, ()> {
@@ -146,7 +164,7 @@ impl<ServerType> Config<ServerType, ()> {
     }
 }
 
-impl<ServerType, AcceptorType: Clone> Clone for Config<ServerType, AcceptorType> {
+impl<ServerType: ?Sized, AcceptorType: Clone> Clone for Config<ServerType, AcceptorType> {
     fn clone(&self) -> Self {
         Self {
             acceptor: self.acceptor.clone(),
@@ -157,12 +175,27 @@ impl<ServerType, AcceptorType: Clone> Clone for Config<ServerType, AcceptorType>
             stopper: self.stopper.clone(),
             counter: self.counter.clone(),
             register_signals: self.register_signals,
+            max_connections: self.max_connections,
         }
     }
 }
 
 impl<ServerType> Default for Config<ServerType, ()> {
     fn default() -> Self {
+        #[cfg(unix)]
+        let max_connections = {
+            use std::convert::TryInto;
+            rlimit::getrlimit(rlimit::Resource::NOFILE)
+                .ok()
+                .and_then(|(soft, _hard)| soft.try_into().ok())
+                .map(|limit: usize| 3 * limit / 4)
+        };
+
+        #[cfg(not(unix))]
+        let max_connections = None;
+
+        log::debug!("using max connections of {:?}", max_connections);
+
         Self {
             acceptor: (),
             port: None,
@@ -172,6 +205,7 @@ impl<ServerType> Default for Config<ServerType, ()> {
             stopper: Stopper::new(),
             counter: CloneCounter::new(),
             register_signals: cfg!(unix),
+            max_connections,
         }
     }
 }
