@@ -1,6 +1,7 @@
 #![deny(
+    clippy::dbg_macro,
     missing_copy_implementations,
-    missing_crate_level_docs,
+    rustdoc::missing_crate_level_docs,
     missing_debug_implementations,
     missing_docs,
     nonstandard_style,
@@ -36,3 +37,6 @@ pub use config_ext::ConfigExt;
 
 mod server;
 pub use server::Server;
+
+mod binding;
+pub use binding::Binding;
